#include <iostream>
#include <sstream>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_math.h>

// macro definitions
#define MAR 'l' //character that identifies a locus in the data
#define PLOIDY 2 // number of allele copies per locus, may not work when != 2

extern gsl_rng * r;
// function declarations
using namespace std;


// struct definitions
struct mat_vec_container_int { 
  gsl_matrix_int * mat; 
  gsl_vector_int * vec; 
}; 

struct mat_container_int { 
  gsl_matrix_int * mat; 
}; 

struct mat_container { 
  gsl_matrix * mat; 
}; 

struct vec_container { 
  gsl_vector * vec; 
}; 

struct datacont{
  mat_container_int * reads;
  gsl_matrix_int * nonzeroalleles;
};

struct paramcont{
  mat_container_int ** genetics;
  mat_container ** prgenetics; // probability of each genotype
  gsl_matrix * popallelefreq;
  gsl_vector * alleleparams;
  gsl_vector * alleleresult;
  gsl_vector_uint * allelecount;
};

struct datadimcont{
  int nind;
  int nloci;
  gsl_vector_int * nallele;
  int maxNallele;
};

struct initcont{
  double alpha; // 1 --  dirichlet parameter
};

// function definitions
void usage(char *);
void initparams(datadimcont * datadim, paramcont * params, initcont * initvals, datacont * data);
int getNloci(string filename);
int getNallele(string filename, gsl_vector_int * nallele);
void getreads(string readfile, datacont * data);
int getNind(string filename);
void mcmcUpdate(datadimcont * datadim, paramcont * params, datacont * data);
void updatePopallelefreq(datadimcont * datadim, paramcont * params);
void updateGenotypes(datadimcont * datadim, paramcont * params, datacont * data);
void writeSamples(datadimcont * datadim, paramcont * params, FILE * fp);
void calcpGenotype(datadimcont * datadim, paramcont * params);
void writeGen(datadimcont * datadim, paramcont * params, FILE * fp, int ctr);
